==> testcafe-13.1.3.tgz: package/lib/fixture_code/func_call_validator.js <==
var astProcessor = require("uglify-js").uglify, SharedConst = require("../shared/const"), Const = require("./const"), Ast = require("./ast"), ErrCodes = require("./err_codes");

function funcCallWalker(ast, handler) {
    var walker = astProcessor.ast_walker();
    walker.with_walkers({
        call: function() {
            var astPath = walker.stack(), line = Ast.getCurrentSrcLineNum(astPath), isDot = Ast.getEntryName(this[1]) === "dot", calleeName = isDot ? this[1][2] : this[1][1], calleeOwnerObj = isDot ? this[1][1][1] : "";
            handler(calleeName, calleeOwnerObj, astPath, line);
        }
    }, function() {
        walker.walk(ast);
    });
}

function isAsyncFuncCall(calleeName, calleeOwnerObj) {
    return (calleeOwnerObj === "window" || calleeOwnerObj === "wnd") && (calleeName === "setTimeout" || calleeName === "setInterval");
}

function isActionFuncCall(calleeName, calleeOwnerObj) {
    return calleeOwnerObj === Const.ACTIONS_OWNER_OBJECT_IDENTIFIER && SharedConst.ACTION_FUNC_NAMES.indexOf(calleeName) > -1;
}

exports.validate = function(ast, filename, errBuff, isSharedCode) {
    funcCallWalker(ast, function(calleeName, calleeOwnerObj, astPath, line) {
        if (isActionFuncCall(calleeName, calleeOwnerObj)) {
            if (isSharedCode) {
                errBuff.push({
                    code: ErrCodes.ACTION_FUNC_CALL_IN_SHARED_CODE,
                    filename: filename,
                    line: line
                });
            } else {
                if (!Ast.isPathMatch(Const.ACTION_FUNC_AST_PATH, astPath, true)) {
                    errBuff.push({
                        code: ErrCodes.ACTION_FUNC_IS_NOT_A_LAST_ENTRY,
                        filename: filename,
                        line: line
                    });
                }
            }
        }
        if (isAsyncFuncCall(calleeName, calleeOwnerObj)) {
            errBuff.push({
                code: ErrCodes.ASYNC_FUNC_CALL,
                filename: filename,
                line: line
            });
        }
    });
};

==> testcafe-14.1.3.tgz: package/lib/fixture_code/func_call_validator.js <==
var astProcessor = require("uglify-js").uglify, SharedConst = require("../shared/const"), Common = require("./common"), Ast = require("./ast"), ErrCodes = require("./err_codes");

function funcCallWalker(ast, handler) {
    var walker = astProcessor.ast_walker();
    walker.with_walkers({
        call: function() {
            var astPath = walker.stack(), line = Ast.getCurrentSrcLineNum(astPath), isDot = Ast.getEntryName(this[1]) === "dot", calleeName = isDot ? this[1][2] : this[1][1], calleeOwnerObj = isDot ? this[1][1][1] : "";
            handler(calleeName, calleeOwnerObj, astPath, line);
        }
    }, function() {
        walker.walk(ast);
    });
}

function isAsyncFuncCall(calleeName, calleeOwnerObj) {
    return (calleeOwnerObj === "window" || calleeOwnerObj === "wnd") && (calleeName === "setTimeout" || calleeName === "setInterval");
}

function isActionFuncCall(calleeName, calleeOwnerObj) {
    return calleeOwnerObj === Common.ACTIONS_OWNER_OBJECT_IDENTIFIER && SharedConst.ACTION_FUNC_NAMES.indexOf(calleeName) > -1;
}

exports.validate = function(ast, filename, errBuff, isSharedCode) {
    funcCallWalker(ast, function(calleeName, calleeOwnerObj, astPath, line) {
        if (isActionFuncCall(calleeName, calleeOwnerObj)) {
            if (isSharedCode) {
                errBuff.push({
                    code: ErrCodes.ACTION_FUNC_CALL_IN_SHARED_CODE,
                    filename: filename,
                    line: line
                });
            } else {
                if (!Ast.isPathMatch(Common.ACTION_FUNC_AST_PATH, astPath, true)) {
                    errBuff.push({
                        code: ErrCodes.ACTION_FUNC_IS_NOT_A_LAST_ENTRY,
                        filename: filename,
                        line: line
                    });
                }
            }
        }
        if (!isSharedCode && isAsyncFuncCall(calleeName, calleeOwnerObj)) {
            errBuff.push({
                code: ErrCodes.ASYNC_FUNC_CALL,
                filename: filename,
                line: line
            });
        }
    });
};
